/* $Id$
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 */

/// @file Medline_si_.hpp
/// Data storage class.
///
/// This file was generated by application DATATOOL
/// using the following specifications:
/// 'medline.asn'.
///
/// ATTENTION:
///   Don't edit or commit this file into CVS as this file will
///   be overridden (by DATATOOL) without warning!

#ifndef OBJECTS_MEDLINE_MEDLINE_SI_BASE_HPP
#define OBJECTS_MEDLINE_MEDLINE_SI_BASE_HPP

// standard includes
#include <serial/serialbase.hpp>

// generated includes
#include <string>

BEGIN_NCBI_SCOPE

#ifndef BEGIN_objects_SCOPE
#  define BEGIN_objects_SCOPE BEGIN_SCOPE(objects)
#  define END_objects_SCOPE END_SCOPE(objects)
#endif
BEGIN_objects_SCOPE // namespace ncbi::objects::


// generated classes

/////////////////////////////////////////////////////////////////////////////
/// medline cross reference records
class NCBI_MEDLINE_EXPORT CMedline_si_Base : public CSerialObject
{
    typedef CSerialObject Tparent;
public:
    // constructor
    CMedline_si_Base(void);
    // destructor
    virtual ~CMedline_si_Base(void);

    // type info
    DECLARE_INTERNAL_TYPE_INFO();

    /// type of xref
    enum EType {
        eType_ddbj      =  1,  ///< DNA Data Bank of Japan
        eType_carbbank  =  2,  ///< Carbohydrate Structure Database
        eType_embl      =  3,  ///< EMBL Data Library
        eType_hdb       =  4,  ///< Hybridoma Data Bank
        eType_genbank   =  5,  ///< GenBank
        eType_hgml      =  6,  ///< Human Gene Map Library
        eType_mim       =  7,  ///< Mendelian Inheritance in Man
        eType_msd       =  8,  ///< Microbial Strains Database
        eType_pdb       =  9,  ///< Protein Data Bank (Brookhaven)
        eType_pir       = 10,  ///< Protein Identification Resource
        eType_prfseqdb  = 11,  ///< Protein Research Foundation (Japan)
        eType_psd       = 12,  ///< Protein Sequence Database (Japan)
        eType_swissprot = 13,  ///< SwissProt
        eType_gdb       = 14  ///< Genome Data Base
    };
    
    /// Access to EType's attributes (values, names) as defined in spec
    static const NCBI_NS_NCBI::CEnumeratedTypeValues* ENUM_METHOD_NAME(EType)(void);
    
    // types
    typedef EType TType;
    typedef string TCit;

    // getters
    // setters

    /// mandatory
    /// typedef EType TType
    ///  Check whether the Type data member has been assigned a value.
    bool IsSetType(void) const;
    /// Check whether it is safe or not to call GetType method.
    bool CanGetType(void) const;
    void ResetType(void);
    TType GetType(void) const;
    void SetType(TType value);
    TType& SetType(void);

    /// the citation/accession number
    /// optional
    /// typedef string TCit
    ///  Check whether the Cit data member has been assigned a value.
    bool IsSetCit(void) const;
    /// Check whether it is safe or not to call GetCit method.
    bool CanGetCit(void) const;
    void ResetCit(void);
    const TCit& GetCit(void) const;
    void SetCit(const TCit& value);
    TCit& SetCit(void);

    /// Reset the whole object
    virtual void Reset(void);


private:
    // Prohibit copy constructor and assignment operator
    CMedline_si_Base(const CMedline_si_Base&);
    CMedline_si_Base& operator=(const CMedline_si_Base&);

    // data
    Uint4 m_set_State[1];
    TType m_Type;
    TCit m_Cit;
};






///////////////////////////////////////////////////////////
///////////////////// inline methods //////////////////////
///////////////////////////////////////////////////////////
inline
bool CMedline_si_Base::IsSetType(void) const
{
    return ((m_set_State[0] & 0x3) != 0);
}

inline
bool CMedline_si_Base::CanGetType(void) const
{
    return IsSetType();
}

inline
void CMedline_si_Base::ResetType(void)
{
    m_Type = (EType)(0);
    m_set_State[0] &= ~0x3;
}

inline
CMedline_si_Base::TType CMedline_si_Base::GetType(void) const
{
    if (!CanGetType()) {
        ThrowUnassigned(0);
    }
    return m_Type;
}

inline
void CMedline_si_Base::SetType(CMedline_si_Base::TType value)
{
    m_Type = value;
    m_set_State[0] |= 0x3;
}

inline
CMedline_si_Base::TType& CMedline_si_Base::SetType(void)
{
#ifdef _DEBUG
    if (!IsSetType()) {
        memset(&m_Type,ms_UnassignedByte,sizeof(m_Type));
    }
#endif
    m_set_State[0] |= 0x1;
    return m_Type;
}

inline
bool CMedline_si_Base::IsSetCit(void) const
{
    return ((m_set_State[0] & 0xc) != 0);
}

inline
bool CMedline_si_Base::CanGetCit(void) const
{
    return IsSetCit();
}

inline
const CMedline_si_Base::TCit& CMedline_si_Base::GetCit(void) const
{
    if (!CanGetCit()) {
        ThrowUnassigned(1);
    }
    return m_Cit;
}

inline
void CMedline_si_Base::SetCit(const CMedline_si_Base::TCit& value)
{
    m_Cit = value;
    m_set_State[0] |= 0xc;
}

inline
CMedline_si_Base::TCit& CMedline_si_Base::SetCit(void)
{
#ifdef _DEBUG
    if (!IsSetCit()) {
        m_Cit = ms_UnassignedStr;
    }
#endif
    m_set_State[0] |= 0x4;
    return m_Cit;
}

///////////////////////////////////////////////////////////
////////////////// end of inline methods //////////////////
///////////////////////////////////////////////////////////





END_objects_SCOPE // namespace ncbi::objects::

END_NCBI_SCOPE


#endif // OBJECTS_MEDLINE_MEDLINE_SI_BASE_HPP
