/* $Id$
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 */

/// @file Numbering_.hpp
/// Data storage class.
///
/// This file was generated by application DATATOOL
/// using the following specifications:
/// 'seq.asn'.
///
/// ATTENTION:
///   Don't edit or commit this file into CVS as this file will
///   be overridden (by DATATOOL) without warning!

#ifndef OBJECTS_SEQ_NUMBERING_BASE_HPP
#define OBJECTS_SEQ_NUMBERING_BASE_HPP

// standard includes
#include <serial/serialbase.hpp>
BEGIN_NCBI_SCOPE

#ifndef BEGIN_objects_SCOPE
#  define BEGIN_objects_SCOPE BEGIN_SCOPE(objects)
#  define END_objects_SCOPE END_SCOPE(objects)
#endif
BEGIN_objects_SCOPE // namespace ncbi::objects::


// forward declarations
class CNum_cont;
class CNum_enum;
class CNum_real;
class CNum_ref;


// generated classes

/////////////////////////////////////////////////////////////////////////////
/// any display numbering system
class NCBI_SEQ_EXPORT CNumbering_Base : public CSerialObject
{
    typedef CSerialObject Tparent;
public:
    // constructor
    CNumbering_Base(void);
    // destructor
    virtual ~CNumbering_Base(void);

    // type info
    DECLARE_INTERNAL_TYPE_INFO();


    /// Choice variants.
    enum E_Choice {
        e_not_set = 0,  ///< No variant selected
        e_Cont,         ///< continuous numbering
        e_Enum,         ///< enumerated names for residues
        e_Ref,          ///< by reference to another sequence
        e_Real          ///< supports mapping to a float system
    };
    /// Maximum+1 value of the choice variant enumerator.
    enum E_ChoiceStopper {
        e_MaxChoice = 5 ///< == e_Real+1
    };

    /// Reset the whole object
    virtual void Reset(void);

    /// Reset the selection (set it to e_not_set).
    virtual void ResetSelection(void);

    /// Which variant is currently selected.
    E_Choice Which(void) const;

    /// Verify selection, throw exception if it differs from the expected.
    void CheckSelected(E_Choice index) const;

    /// Throw 'InvalidSelection' exception.
    NCBI_NORETURN void ThrowInvalidSelection(E_Choice index) const;

    /// Retrieve selection name (for diagnostic purposes).
    static string SelectionName(E_Choice index);

    /// Select the requested variant if needed.
    void Select(E_Choice index, EResetVariant reset = eDoResetVariant);
    /// Select the requested variant if needed,
    /// allocating CObject variants from memory pool.
    void Select(E_Choice index,
                EResetVariant reset,
                CObjectMemoryPool* pool);

    // types
    typedef CNum_cont TCont;
    typedef CNum_enum TEnum;
    typedef CNum_ref TRef;
    typedef CNum_real TReal;

    // getters
    // setters

    // typedef CNum_cont TCont
    bool IsCont(void) const;
    const TCont& GetCont(void) const;
    TCont& SetCont(void);
    void SetCont(TCont& value);

    // typedef CNum_enum TEnum
    bool IsEnum(void) const;
    const TEnum& GetEnum(void) const;
    TEnum& SetEnum(void);
    void SetEnum(TEnum& value);

    // typedef CNum_ref TRef
    bool IsRef(void) const;
    const TRef& GetRef(void) const;
    TRef& SetRef(void);
    void SetRef(TRef& value);

    // typedef CNum_real TReal
    bool IsReal(void) const;
    const TReal& GetReal(void) const;
    TReal& SetReal(void);
    void SetReal(TReal& value);


private:
    // copy constructor and assignment operator
    CNumbering_Base(const CNumbering_Base& );
    CNumbering_Base& operator=(const CNumbering_Base& );
    // choice state
    E_Choice m_choice;
    // helper methods
    void DoSelect(E_Choice index, CObjectMemoryPool* pool = 0);

    static const char* const sm_SelectionNames[];
    // data
    NCBI_NS_NCBI::CSerialObject *m_object;
};






///////////////////////////////////////////////////////////
///////////////////// inline methods //////////////////////
///////////////////////////////////////////////////////////
inline
CNumbering_Base::E_Choice CNumbering_Base::Which(void) const
{
    return m_choice;
}

inline
void CNumbering_Base::CheckSelected(E_Choice index) const
{
    if ( m_choice != index )
        ThrowInvalidSelection(index);
}

inline
void CNumbering_Base::Select(E_Choice index, NCBI_NS_NCBI::EResetVariant reset, NCBI_NS_NCBI::CObjectMemoryPool* pool)
{
    if ( reset == NCBI_NS_NCBI::eDoResetVariant || m_choice != index ) {
        if ( m_choice != e_not_set )
            ResetSelection();
        DoSelect(index, pool);
    }
}

inline
void CNumbering_Base::Select(E_Choice index, NCBI_NS_NCBI::EResetVariant reset)
{
    Select(index, reset, 0);
}

inline
bool CNumbering_Base::IsCont(void) const
{
    return m_choice == e_Cont;
}

inline
bool CNumbering_Base::IsEnum(void) const
{
    return m_choice == e_Enum;
}

inline
bool CNumbering_Base::IsRef(void) const
{
    return m_choice == e_Ref;
}

inline
bool CNumbering_Base::IsReal(void) const
{
    return m_choice == e_Real;
}

///////////////////////////////////////////////////////////
////////////////// end of inline methods //////////////////
///////////////////////////////////////////////////////////





END_objects_SCOPE // namespace ncbi::objects::

END_NCBI_SCOPE


#endif // OBJECTS_SEQ_NUMBERING_BASE_HPP
