/* $Id$
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 */

/// @file Product_pos_.hpp
/// Data storage class.
///
/// This file was generated by application DATATOOL
/// using the following specifications:
/// 'seqalign.asn'.
///
/// ATTENTION:
///   Don't edit or commit this file into CVS as this file will
///   be overridden (by DATATOOL) without warning!

#ifndef OBJECTS_SEQALIGN_PRODUCT_POS_BASE_HPP
#define OBJECTS_SEQALIGN_PRODUCT_POS_BASE_HPP

// standard includes
#include <serial/serialbase.hpp>
BEGIN_NCBI_SCOPE

#ifndef BEGIN_objects_SCOPE
#  define BEGIN_objects_SCOPE BEGIN_SCOPE(objects)
#  define END_objects_SCOPE END_SCOPE(objects)
#endif
BEGIN_objects_SCOPE // namespace ncbi::objects::


// forward declarations
class CProt_pos;


// generated classes

/////////////////////////////////////////////////////////////////////////////
class NCBI_SEQALIGN_EXPORT CProduct_pos_Base : public CSerialObject
{
    typedef CSerialObject Tparent;
public:
    // constructor
    CProduct_pos_Base(void);
    // destructor
    virtual ~CProduct_pos_Base(void);

    // type info
    DECLARE_INTERNAL_TYPE_INFO();


    /// Choice variants.
    enum E_Choice {
        e_not_set = 0,  ///< No variant selected
        e_Nucpos,
        e_Protpos
    };
    /// Maximum+1 value of the choice variant enumerator.
    enum E_ChoiceStopper {
        e_MaxChoice = 3 ///< == e_Protpos+1
    };

    /// Reset the whole object
    virtual void Reset(void);

    /// Reset the selection (set it to e_not_set).
    virtual void ResetSelection(void);

    /// Which variant is currently selected.
    E_Choice Which(void) const;

    /// Verify selection, throw exception if it differs from the expected.
    void CheckSelected(E_Choice index) const;

    /// Throw 'InvalidSelection' exception.
    NCBI_NORETURN void ThrowInvalidSelection(E_Choice index) const;

    /// Retrieve selection name (for diagnostic purposes).
    static string SelectionName(E_Choice index);

    /// Select the requested variant if needed.
    void Select(E_Choice index, EResetVariant reset = eDoResetVariant);
    /// Select the requested variant if needed,
    /// allocating CObject variants from memory pool.
    void Select(E_Choice index,
                EResetVariant reset,
                CObjectMemoryPool* pool);

    // types
    typedef TSeqPos TNucpos;
    typedef CProt_pos TProtpos;

    // getters
    // setters

    // typedef TSeqPos TNucpos
    bool IsNucpos(void) const;
    TNucpos GetNucpos(void) const;
    TNucpos& SetNucpos(void);
    void SetNucpos(TNucpos value);

    // typedef CProt_pos TProtpos
    bool IsProtpos(void) const;
    const TProtpos& GetProtpos(void) const;
    TProtpos& SetProtpos(void);
    void SetProtpos(TProtpos& value);


private:
    // copy constructor and assignment operator
    CProduct_pos_Base(const CProduct_pos_Base& );
    CProduct_pos_Base& operator=(const CProduct_pos_Base& );
    // choice state
    E_Choice m_choice;
    // helper methods
    void DoSelect(E_Choice index, CObjectMemoryPool* pool = 0);

    static const char* const sm_SelectionNames[];
    // data
    union {
        TNucpos m_Nucpos;
        NCBI_NS_NCBI::CSerialObject *m_object;
    };
};






///////////////////////////////////////////////////////////
///////////////////// inline methods //////////////////////
///////////////////////////////////////////////////////////
inline
CProduct_pos_Base::E_Choice CProduct_pos_Base::Which(void) const
{
    return m_choice;
}

inline
void CProduct_pos_Base::CheckSelected(E_Choice index) const
{
    if ( m_choice != index )
        ThrowInvalidSelection(index);
}

inline
void CProduct_pos_Base::Select(E_Choice index, NCBI_NS_NCBI::EResetVariant reset, NCBI_NS_NCBI::CObjectMemoryPool* pool)
{
    if ( reset == NCBI_NS_NCBI::eDoResetVariant || m_choice != index ) {
        if ( m_choice != e_not_set )
            ResetSelection();
        DoSelect(index, pool);
    }
}

inline
void CProduct_pos_Base::Select(E_Choice index, NCBI_NS_NCBI::EResetVariant reset)
{
    Select(index, reset, 0);
}

inline
bool CProduct_pos_Base::IsNucpos(void) const
{
    return m_choice == e_Nucpos;
}

inline
CProduct_pos_Base::TNucpos CProduct_pos_Base::GetNucpos(void) const
{
    CheckSelected(e_Nucpos);
    return m_Nucpos;
}

inline
CProduct_pos_Base::TNucpos& CProduct_pos_Base::SetNucpos(void)
{
    Select(e_Nucpos, NCBI_NS_NCBI::eDoNotResetVariant);
    return m_Nucpos;
}

inline
void CProduct_pos_Base::SetNucpos(CProduct_pos_Base::TNucpos value)
{
    Select(e_Nucpos, NCBI_NS_NCBI::eDoNotResetVariant);
    m_Nucpos = value;
}

inline
bool CProduct_pos_Base::IsProtpos(void) const
{
    return m_choice == e_Protpos;
}

///////////////////////////////////////////////////////////
////////////////// end of inline methods //////////////////
///////////////////////////////////////////////////////////





END_objects_SCOPE // namespace ncbi::objects::

END_NCBI_SCOPE


#endif // OBJECTS_SEQALIGN_PRODUCT_POS_BASE_HPP
